@@ -3,14 +3,11 @@
  * This file is a part of the Links program, released under GPL
  */
 
-#if defined(__BEOS__)
-
-#include "com-defs.h"
+#if defined(__BEOS__) || defined(__HAIKU__)
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -85,10 +82,10 @@ int be_accept(int s, struct sockaddr *sa, int *sal)
 int be_bind(int s, struct sockaddr *sa, int sal)
 {
 	/*struct sockaddr_in *sin = (struct sockaddr_in *)sa;
-	if (!ntohs(sin->sin_port)) {
+	if (!sin->sin_port) {
 		int i;
 		for (i = 16384; i < 49152; i++) {
-			sin->sin_port = htons(i);
+			sin->sin_port = i;
 			if (!be_bind(s, sa, sal)) return 0;
 		}
 		return -1;
@@ -100,7 +97,7 @@ int be_bind(int s, struct sockaddr *sa, int sal)
 
 #define PIPE_RETRIES	10
 
-int be_pipe(int fd[2])
+int be_pipe(int *fd)
 {
 	int s1, s2, s3, l;
 	struct sockaddr_in sa1, sa2;
@@ -117,7 +114,7 @@ int be_pipe(int fd[2])
 	}
 	memset(&sa1, 0, sizeof(sa1));
 	sa1.sin_family = AF_INET;
-	sa1.sin_port = htons(0);
+	sa1.sin_port = 0;
 	sa1.sin_addr.s_addr = INADDR_ANY;
 	if (be_bind(s1, (struct sockaddr *)&sa1, sizeof(sa1))) {
 		/*perror("bind");*/
@@ -159,7 +156,6 @@ int be_select(int n, struct fd_set *rd, struct fd_set *wr, struct fd_set *exc, s
 {
 	int i, s;
 	struct fd_set d, rrd;
-retry:
 	FD_ZERO(&d);
 	if (!rd) rd = &d;
 	if (!wr) wr = &d;
@@ -173,7 +169,6 @@ retry:
 	FD_ZERO(&rrd);
 	for (i = SHS; i < n; i++) if (FD_ISSET(i, rd)) FD_SET(i - SHS, &rrd);
 	if ((s = select(FD_SETSIZE, &rrd, &d, &d, tm)) < 0) {
-		if (errno == EINTR) goto retry;
 		FD_ZERO(rd);
 		return 0;
 	}
@@ -200,6 +195,8 @@ static int ihpipe[2];
 
 static int inth;
 
+#include <errno.h>
+
 static void input_handle_th(void *p)
 {
 	char c;
@@ -216,7 +213,9 @@ int get_input_handle(void)
 	if ((inth = start_thr(input_handle_th, NULL, "input_thread")) < 0) {
 		closesocket(ihpipe[0]);
 		closesocket(ihpipe[1]);
-		fatal_exit("Can't spawn input thread");
+		error("Can't spawn input thread");
+		fatal_tty_exit();
+		exit(4);
 	}
 	return h = ihpipe[0];
 }
@@ -253,7 +252,9 @@ int get_output_handle(void)
 	if (start_thr(output_handle_th, NULL, "output_thread") < 0) {
 		closesocket(ohpipe[0]);
 		closesocket(ohpipe[1]);
-		fatal_exit("Can't spawn output thread");
+		error("Can't spawn output thread");
+		fatal_tty_exit();
+		exit(4);
 	}
 	return h = ohpipe[1];
 }*/
