@@ -5,31 +5,15 @@
 
 #include "links.h"
 
-#if defined(HAVE_LIBBSD)
-
-static void stat_mode(unsigned char **p, int *l, struct stat *stp)
-{
-	unsigned char bp[12] = "?--------- ";
-	if (stp)
-		strmode(stp->st_mode, cast_char bp);
-#ifndef FS_UNIX_RIGHTS
-	bp[1] = ' ';
-	bp[2] = 0;
-#endif
-	add_to_str(p, l, bp);
-}
-
-#else
-
 #ifdef FS_UNIX_RIGHTS
-static void setrwx(unsigned m, unsigned char *p)
+static void setrwx(int m, unsigned char *p)
 {
-	if (m & S_IRUSR) p[0] = 'r';
-	if (m & S_IWUSR) p[1] = 'w';
-	if (m & S_IXUSR) p[2] = 'x';
+	if(m & S_IRUSR) p[0] = 'r';
+	if(m & S_IWUSR) p[1] = 'w';
+	if(m & S_IXUSR) p[2] = 'x';
 }
 
-static void setst(unsigned m, unsigned char *p)
+static void setst(int m, unsigned char *p)
 {
 #ifdef S_ISUID
 	if (m & S_ISUID) {
@@ -83,7 +67,7 @@ static void stat_mode(unsigned char **p, int *l, struct stat *stp)
 	{
 		unsigned char rwx[10] = "---------";
 		if (stp) {
-			unsigned mode = stp->st_mode;
+			int mode = stp->st_mode;
 			setrwx(mode << 0, &rwx[0]);
 			setrwx(mode << 3, &rwx[3]);
 			setrwx(mode << 6, &rwx[6]);
@@ -95,16 +79,14 @@ static void stat_mode(unsigned char **p, int *l, struct stat *stp)
 	add_chr_to_str(p, l, ' ');
 }
 
-#endif
-
 
 static void stat_links(unsigned char **p, int *l, struct stat *stp)
 {
 #ifdef FS_UNIX_HARDLINKS
 	unsigned char lnk[64];
-	if (!stp) add_to_str(p, l, cast_uchar "    ");
+	if (!stp) add_to_str(p, l, "    ");
 	else {
-		sprintf(cast_char lnk, "%3ld ", (unsigned long)stp->st_nlink);
+		sprintf(lnk, "%3ld ", (unsigned long)stp->st_nlink);
 		add_to_str(p, l, lnk);
 	}
 #endif
@@ -127,7 +109,7 @@ static void stat_user(unsigned char **p, int *l, struct stat *stp, int g)
 	unsigned char *pp;
 	int i;
 	if (!stp) {
-		add_to_str(p, l, cast_uchar "         ");
+		add_to_str(p, l, "         ");
 		return;
 	}
 	id = !g ? stp->st_uid : stp->st_gid;
@@ -135,19 +117,17 @@ static void stat_user(unsigned char **p, int *l, struct stat *stp, int g)
 	if (!g && id == last_uid && last_uid != -1) goto a;
 	if (g && id == last_gid && last_gid != -1) goto a;
 	if (!g) {
-		ENULLLOOP(pwd, getpwuid(id));
-		if (!pwd || !pwd->pw_name) sprintf(cast_char pp, "%d", id);
-		else sprintf(cast_char pp, "%.8s", pwd->pw_name);
+		if (!(pwd = getpwuid(id)) || !pwd->pw_name) sprintf(pp, "%d", id);
+		else sprintf(pp, "%.8s", pwd->pw_name);
 		last_uid = id;
 	} else {
-		ENULLLOOP(grp, getgrgid(id));
-		if (!grp || !grp->gr_name) sprintf(cast_char pp, "%d", id);
-		else sprintf(cast_char pp, "%.8s", grp->gr_name);
+		if (!(grp = getgrgid(id)) || !grp->gr_name) sprintf(pp, "%d", id);
+		else sprintf(pp, "%.8s", grp->gr_name);
 		last_gid = id;
 	}
 	a:
 	add_to_str(p, l, pp);
-	for (i = (int)strlen(cast_const_char pp); i < 8; i++) add_chr_to_str(p, l, ' ');
+	for (i = strlen(pp); i < 8; i++) add_chr_to_str(p, l, ' ');
 	add_chr_to_str(p, l, ' ');
 #endif
 }
@@ -160,9 +140,9 @@ static void stat_size(unsigned char **p, int *l, struct stat *stp)
 	if (!stp) {
 		num[0] = 0;
 	} else {
-		snzprint(num, sizeof num, stp->st_size);
+		snprint(num, sizeof num, stp->st_size);
 	}
-	for (i = (int)strlen(cast_const_char num); i < digits; i++)
+	for (i = strlen(num); i < digits; i++)
 		add_chr_to_str(p, l, ' ');
 	add_to_str(p, l, num);
 	add_chr_to_str(p, l, ' ');
@@ -170,53 +150,43 @@ static void stat_size(unsigned char **p, int *l, struct stat *stp)
 
 static void stat_date(unsigned char **p, int *l, struct stat *stp)
 {
-	time_t current_time;
+	time_t current_time = time(NULL);
 	time_t when;
 	struct tm *when_local;
-	unsigned char *fmt, *e;
+	unsigned char *fmt;
 	unsigned char str[13];
-	static unsigned char fmt1[] = "%b %e  %Y";
-	static unsigned char fmt2[] = "%b %e %H:%M";
 	int wr;
-	current_time = get_absolute_seconds();
 	if (!stp) {
-		wr = 0;
-		goto set_empty;
+		add_to_str(p, l, "             ");
+		return;
 	}
 	when = stp->st_mtime;
 	when_local = localtime(&when);
-	if ((ulonglong)current_time > (ulonglong)when + 6L * 30L * 24L * 60L * 60L ||
-	    (ulonglong)current_time < (ulonglong)when - 60L * 60L) fmt = fmt1;
-	else fmt = fmt2;
+	if (current_time > when + 6L * 30L * 24L * 60L * 60L || 
+	    current_time < when - 60L * 60L) fmt = "%b %e  %Y";
+	else fmt = "%b %e %H:%M";
 #ifdef HAVE_STRFTIME
-again:
-	wr = (int)strftime(cast_char str, 13, cast_const_char fmt, when_local);
-	if (wr && strstr(cast_const_char str, " e ") && ((e = cast_uchar strchr(cast_const_char fmt, 'e')))) {
-		*e = 'd';
-		goto again;
-	}
+	wr = strftime(str, 13, fmt, when_local);
 #else
 	wr = 0;
 #endif
-	set_empty:
 	while (wr < 12) str[wr++] = ' ';
 	str[12] = 0;
 	add_to_str(p, l, str);
 	add_chr_to_str(p, l, ' ');
 }
 
-/* ATTR_NOINLINE is a workaround for a bogus warning on gcc 12 */
-ATTR_NOINLINE static unsigned char *get_filename(unsigned char *url)
+static unsigned char *get_filename(unsigned char *url)
 {
 	unsigned char *p, *m;
 	int ml;
 #ifdef DOS_FS
-	if (url[7] == '/' && strchr(cast_const_char(url + 8), ':')) url++;
+	if (url[7] == '/' && strchr(url + 8, ':')) url++;
 #endif
 	for (p = url + 7; *p && *p != POST_CHAR; p++)
 		;
 	m = init_str(), ml = 0;
-	add_conv_str(&m, &ml, url + 7, (int)(p - url - 7), -2);
+	add_conv_str(&m, &ml, url + 7, p - url - 7, -2);
 	return m;
 }
 
@@ -225,26 +195,23 @@ struct dirs {
 	unsigned char *f;
 };
 
-LIBC_CALLBACK static int comp_de(const void *d1_, const void *d2_)
+LIBC_CALLBACK static int comp_de(struct dirs *d1, struct dirs *d2)
 {
-	const struct dirs *d1 = (const struct dirs *)d1_;
-	const struct dirs *d2 = (const struct dirs *)d2_;
 	if (d1->f[0] == '.' && d1->f[1] == '.' && !d1->f[2]) return -1;
 	if (d2->f[0] == '.' && d2->f[1] == '.' && !d2->f[2]) return 1;
 	if (d1->s[0] == 'd' && d2->s[0] != 'd') return -1;
 	if (d1->s[0] != 'd' && d2->s[0] == 'd') return 1;
-	return strcmp(cast_const_char d1->f, cast_const_char d2->f);
+	return strcmp(d1->f, d2->f);
 }
 
 void file_func(struct connection *c)
 {
 	struct cache_entry *e;
 	unsigned char *file, *name, *head = NULL; /* against warning */
-	int fl, flo;
+	int fl;
 	DIR *d;
 	int h, r;
 	struct stat stt;
-	int rs;
 	if (anonymous) {
 		setcstate(c, S_BAD_URL);
 		abort_connection(c);
@@ -253,8 +220,7 @@ void file_func(struct connection *c)
 	if (!(name = get_filename(c->url))) {
 		setcstate(c, S_OUT_OF_MEM); abort_connection(c); return;
 	}
-	EINTRLOOP(rs, stat(cast_const_char name, &stt));
-	if (rs) {
+	if (stat(name, &stt)) {
 		mem_free(name);
 		setcstate(c, get_error_from_errno(errno)); abort_connection(c); return;
 	}
@@ -262,44 +228,40 @@ void file_func(struct connection *c)
 		mem_free(name);
 		setcstate(c, S_FILE_TYPE); abort_connection(c); return;
 	}
-	h = c_open(name, O_RDONLY | O_NOCTTY);
-	if (h == -1) {
+	if ((h = open(name, O_RDONLY | O_NOCTTY)) == -1) {
 		int er = errno;
-		d = c_opendir(name);
-		if (d) goto dir;
+		if ((d = opendir(name))) goto dir;
 		mem_free(name);
 		setcstate(c, get_error_from_errno(er));
 		abort_connection(c);
 		return;
 	}
+	set_bin(h);
 	if (S_ISDIR(stt.st_mode)) {
 		struct dirs *dir;
 		int dirl;
 		int i;
-		int er;
 		struct dirent *de;
-		d = c_opendir(name);
-		er = errno;
-		EINTRLOOP(rs, close(h));
+		d = opendir(name);
+		close(h);
 		if (!d) {
 			mem_free(name);
-			setcstate(c, get_error_from_errno(er)); abort_connection(c); return;
+			setcstate(c, get_error_from_errno(errno)); abort_connection(c); return;
 		}
 		dir:
 		dir = DUMMY, dirl = 0;
-		if (name[0] && !dir_sep(name[strlen(cast_const_char name) - 1])) {
-			if (!c->cache) {
-				if (get_connection_cache_entry(c)) {
-					mem_free(name);
-					closedir(d);
-					setcstate(c, S_OUT_OF_MEM); abort_connection(c); return;
-				}
-				c->cache->refcount--;
+		if (name[0] && !dir_sep(name[strlen(name) - 1])) {
+			if (get_cache_entry(c->url, &e)) {
+				mem_free(name);
+				closedir(d);
+				setcstate(c, S_OUT_OF_MEM); abort_connection(c); return;
 			}
-			e = c->cache;
+			c->cache = e;
+			e->refcount--;
 			if (e->redirect) mem_free(e->redirect);
 			e->redirect = stracpy(c->url);
-			add_to_strn(&e->redirect, cast_uchar "/");
+			e->redirect_get = 1;
+			add_to_strn(&e->redirect, "/");
 			mem_free(name);
 			closedir(d);
 			goto end;
@@ -310,45 +272,30 @@ void file_func(struct connection *c)
 #endif
 		file = init_str();
 		fl = 0;
-		add_to_str(&file, &fl, cast_uchar "<html><head><title>");
-		flo = fl;
-		add_conv_str(&file, &fl, name, (int)strlen(cast_const_char name), -1);
-		convert_file_charset(&file, &fl, flo);
-		add_to_str(&file, &fl, cast_uchar "</title></head><body><h2>Directory ");
-		flo = fl;
-		add_conv_str(&file, &fl, name, (int)strlen(cast_const_char name), -1);
-		convert_file_charset(&file, &fl, flo);
-		add_to_str(&file, &fl, cast_uchar "</h2>\n<pre>");
-		while (1) {
+		add_to_str(&file, &fl, "<html><head><title>");
+		add_conv_str(&file, &fl, name, strlen(name), -1);
+		add_to_str(&file, &fl, "</title></head><body><h2>Directory ");
+		add_conv_str(&file, &fl, name, strlen(name), -1);
+		add_to_str(&file, &fl, "</h2><pre>");
+		while ((de = (void *)readdir(d))) {
 			struct stat stt, *stp;
 			unsigned char **p;
 			int l;
 			unsigned char *n;
-			ENULLLOOP(de, (void *)readdir(d));
-			if (!de) break;
-			if (!strcmp(cast_const_char de->d_name, ".")) continue;
-			if (!strcmp(cast_const_char de->d_name, "..")) {
-				unsigned char *n = name;
-#if defined(DOS_FS) || defined(SPAD)
-				unsigned char *nn = cast_uchar strchr(cast_const_char n, ':');
-				if (nn) n = nn + 1;
-#endif
-				if (!n[strspn(cast_const_char n, dir_sep('\\') ? "/\\" : "/")])
-					continue;
-			}
+			if (!strcmp(de->d_name, ".")) continue;
 			if ((unsigned)dirl > MAXINT / sizeof(struct dirs) - 1) overalloc();
 			dir = mem_realloc(dir, (dirl + 1) * sizeof(struct dirs));
-			dir[dirl].f = stracpy(cast_uchar de->d_name);
+			dir[dirl].f = stracpy(de->d_name);
 			*(p = &dir[dirl++].s) = init_str();
 			l = 0;
 			n = stracpy(name);
-			add_to_strn(&n, cast_uchar de->d_name);
+			add_to_strn(&n, de->d_name);
 #ifdef FS_UNIX_SOFTLINKS
-			EINTRLOOP(rs, lstat(cast_const_char n, &stt));
+			if (lstat(n, &stt))
 #else
-			EINTRLOOP(rs, stat(cast_const_char n, &stt));
+			if (stat(n, &stt))
 #endif
-			if (rs) stp = NULL;
+			     stp = NULL;
 			else stp = &stt;
 			mem_free(n);
 			stat_mode(p, &l, stp);
@@ -359,7 +306,7 @@ void file_func(struct connection *c)
 			stat_date(p, &l, stp);
 		}
 		closedir(d);
-		if (dirl) qsort(dir, dirl, sizeof(struct dirs), (int (*)(const void *, const void *))comp_de);
+		if (dirl) qsort(dir, dirl, sizeof(struct dirs), (int(*)(const void *, const void *))comp_de);
 		for (i = 0; i < dirl; i++) {
 			unsigned char *lnk = NULL;
 #ifdef FS_UNIX_SOFTLINKS
@@ -374,7 +321,7 @@ void file_func(struct connection *c)
 					size += ALLOC_GR;
 					if ((unsigned)size > MAXINT) overalloc();
 					buf = mem_alloc(size);
-					EINTRLOOP(r, (int)readlink(cast_const_char n, cast_char buf, size));
+					r = readlink(n, buf, size);
 				} while (r == size);
 				if (r == -1) goto yyy;
 				buf[r] = 0;
@@ -386,94 +333,67 @@ void file_func(struct connection *c)
 				mem_free(n);
 			}
 #endif
-			/*add_to_str(&file, &fl, cast_uchar "   ");*/
+			/*add_to_str(&file, &fl, "   ");*/
 			add_to_str(&file, &fl, dir[i].s);
-			add_to_str(&file, &fl, cast_uchar "<a href=\"./");
-			add_conv_str(&file, &fl, dir[i].f, (int)strlen(cast_const_char dir[i].f), 1);
-			if (dir[i].s[0] == 'd') add_chr_to_str(&file, &fl, '/');
+			add_to_str(&file, &fl, "<a href=\"./");
+			add_conv_str(&file, &fl, dir[i].f, strlen(dir[i].f), 1);
+			if (dir[i].s[0] == 'd') add_to_str(&file, &fl, "/");
 			else if (lnk) {
 				struct stat st;
 				unsigned char *n = stracpy(name);
 				add_to_strn(&n, dir[i].f);
-				EINTRLOOP(rs, stat(cast_const_char n, &st));
-				if (!rs) if (S_ISDIR(st.st_mode)) add_chr_to_str(&file, &fl, '/');
+				if (!stat(n, &st)) if (S_ISDIR(st.st_mode)) add_to_str(&file, &fl, "/");
 				mem_free(n);
 			}
-			add_to_str(&file, &fl, cast_uchar "\">");
-			/*if (dir[i].s[0] == 'd') add_to_str(&file, &fl, cast_uchar "<font color=\"yellow\">");*/
-			flo = fl;
-			add_conv_str(&file, &fl, dir[i].f, (int)strlen(cast_const_char dir[i].f), 0);
-			convert_file_charset(&file, &fl, flo);
-			/*if (dir[i].s[0] == 'd') add_to_str(&file, &fl, cast_uchar "</font>");*/
-			add_to_str(&file, &fl, cast_uchar "</a>");
+			add_to_str(&file, &fl, "\">");
+			/*if (dir[i].s[0] == 'd') add_to_str(&file, &fl, "<font color=\"yellow\">");*/
+			add_conv_str(&file, &fl, dir[i].f, strlen(dir[i].f), 0);
+			/*if (dir[i].s[0] == 'd') add_to_str(&file, &fl, "</font>");*/
+			add_to_str(&file, &fl, "</a>");
 			if (lnk) {
-				add_to_str(&file, &fl, cast_uchar " -> ");
+				add_to_str(&file, &fl, " -> ");
 				add_to_str(&file, &fl, lnk);
 				mem_free(lnk);
 			}
-			add_to_str(&file, &fl, cast_uchar "\n");
+			add_to_str(&file, &fl, "\n");
 		}
 		mem_free(name);
 		for (i = 0; i < dirl; i++) mem_free(dir[i].s), mem_free(dir[i].f);
 		mem_free(dir);
-		add_to_str(&file, &fl, cast_uchar "</pre></body></html>\n");
-		head = stracpy(cast_uchar "\r\nContent-Type: text/html\r\n");
+		add_to_str(&file, &fl, "</pre></body></html>\n");
+		head = stracpy("\r\nContent-Type: text/html\r\n");
 	} else {
 		mem_free(name);
-		if (
-#ifndef __WATCOMC__
-		    stt.st_size < 0 ||
-#endif
-		    stt.st_size > MAXINT) {
-			EINTRLOOP(rs, close(h));
+		if (stt.st_size > MAXINT) {
+			close(h);
 			setcstate(c, S_LARGE_FILE); abort_connection(c);
 			return;
 		}
 		/* + !stt.st_size is there because of bug in Linux. Read returns
 		   -EACCES when reading 0 bytes to invalid address */
-		file = mem_alloc_mayfail((size_t)stt.st_size + !stt.st_size);
-		if (!file) {
-			EINTRLOOP(rs, close(h));
-			setcstate(c, S_OUT_OF_MEM);
-			abort_connection(c); return;
-		}
-		if ((r = hard_read(h, file, (int)stt.st_size))
-#ifdef OPENVMS
-			< 0
-#else
-			!= stt.st_size
-#endif
-			) {
-			mem_free(file);
-			EINTRLOOP(rs, close(h));
+		file = mem_alloc(stt.st_size + !stt.st_size);
+		if ((r = hard_read(h, file, stt.st_size)) != stt.st_size) {
+			mem_free(file); close(h);
 			setcstate(c, r == -1 ? get_error_from_errno(errno) : S_FILE_ERROR);
 			abort_connection(c); return;
 		}
-		fl = r;
-		EINTRLOOP(rs, close(h));
-		head = stracpy(cast_uchar "");
+		close(h);
+		fl = stt.st_size;
+		head = stracpy("");
 	}
-	if (!c->cache) {
-		if (get_connection_cache_entry(c)) {
-			mem_free(file);
-			mem_free(head);
-			setcstate(c, S_OUT_OF_MEM); abort_connection(c); return;
-		}
-		c->cache->refcount--;
+	if (get_cache_entry(c->url, &e)) {
+		mem_free(file);
+		setcstate(c, S_OUT_OF_MEM); abort_connection(c); return;
 	}
-	e = c->cache;
+	c->cache = e;
+	e->refcount--;
 	if (e->head) mem_free(e->head);
 	e->head = head;
-	if ((r = add_fragment(e, 0, file, fl)) < 0) {
-		mem_free(file);
-		setcstate(c, r);
-		abort_connection(c);
-		return;
-	}
+	add_fragment(e, 0, file, fl);
 	truncate_entry(e, fl, 1);
 	mem_free(file);
 	end:
 	c->cache->incomplete = 0;
-	setcstate(c, S__OK);
+	setcstate(c, S_OK);
 	abort_connection(c);
 }
