@@ -10,46 +10,12 @@
 
 #ifdef G
 
+#ifdef HAVE_MATH_H
+#include <math.h>
+#endif
+
 #include "links.h"
 
-struct gif_table_entry {
-	unsigned char end_char;
-	unsigned char garbage; /* This has nothing common to do with code table:
-			this is temporarily used for reverting strings :-) */
-	short pointer; /* points onto another entry in table, number 0...4095.
-			number -1 means it end there, the end_char is the last
-			number -2 means that this entry is no filled in yet. */
-};
-
-struct gif_decoder {
-	unsigned char *color_map; /* NULL if no color map, otherwise a block of 768 bytes, red, green, blue,
-				     in sRGB, describing color slots 0...255. */
-	int state; /* State of the automatus finitus recognizing the GIF
-		    * format.  0 is initial. */
-	/* Image width, height, bits per pixel, bytes per line, number of bit planes */
-	int im_width;
-	int im_height;
-	int im_bpp; /* Bits per pixel (in codestream) */
-	int code_size;
-	int initial_code_size;
-	int remains; /* Used to skip unwanted blocks in raster data */
-	struct gif_table_entry table[4096]; /* NULL when not present */
-	unsigned char *actual_line; /* Points to actual line in goi->buffer */
-	unsigned char tbuf[16]; /* For remembering headers and similar things. */
-	int tlen; /* 0 in the beginning . tbuf length */
-	int xoff, yoff;
-	int interl_dist;
-	int bits_read; /* How many bits are already read from the symbol
-			* Currently being read */
-	int last_code; /* This is somehow used in the decompression algorithm */
-	int read_code;
-	int CC;
-	int EOI;
-	int table_pos;
-	int first_code;
-	int transparent;
-};
-
 /****************************** Functions *************************************/
 
 /* Takes the argument from global_cimg. Does not free the gif_decoder
@@ -72,7 +38,7 @@ static void alloc_color_map(int colors)
  deco->color_map=mem_alloc(colors*3*sizeof(*(deco->color_map)));
 }
 
-/*
+/* 
    Initialize code table: construct codes 0...(1<<code_size)-1 with values
    0...(1<<code_size)-1 Codes (1<<code_size) and (1<<code_size)+1 are
    left intact -- they are of no use.
@@ -95,7 +61,7 @@ static void init_table(void)
  for (i=0;i<1<<deco->code_size;i++)
  {
   deco->table[i].pointer=-1;
-  deco->table[i].end_char=(unsigned char)i;
+  deco->table[i].end_char=i;
  }
  /* Here i=1<<code_size. */
  deco->CC=i;
@@ -103,17 +69,17 @@ static void init_table(void)
  deco->table_pos=i+2;
  for (;i<4096;i++)
  {
-  deco->table[i].pointer=-2;
+  deco->table[i].pointer=-2; 
  }
  deco->code_size++;
  deco->last_code=0;
 }
 
-/*
+/* 
    Outputs a single pixel.
    if end_callback_hit gets set, do not send any more data in.
 */
-static inline void
+static inline void 
 output_pixel(int c)
 {
 	struct gif_decoder *deco;
@@ -133,21 +99,21 @@ output_pixel(int c)
 		deco->xoff=0;
 		global_cimg->rows_added=1;
 		if (deco->interl_dist==1)
-		{
+		{ 
 			if (global_cimg->strip_optimized){
 				buffer_to_bitmap_incremental(cimg
 					,deco->actual_line, 1, deco->yoff,
 					cimg->dregs, 1);
-			}else{
-				deco->actual_line+=deco->im_width*3;
-			}
+	  		}else{
+	  			deco->actual_line+=deco->im_width*3;
+	  		}
 			deco->yoff++;
-		}else{
+  		}else{
 			int skip=deco->interl_dist&15;
-			int n=(deco->interl_dist==24)
+   			int n=(deco->interl_dist==24)
 				?8:(deco->interl_dist>>1);
 			unsigned char *ptr;
-			int y;
+   			int y;
 
 			ptr=deco->actual_line+deco->im_width*3;
 			for (y=deco->yoff+1;y<deco->im_height
@@ -169,12 +135,12 @@ output_pixel(int c)
 			}else{
 				deco->interl_dist=(deco->interl_dist==24)
 					?8:(deco->interl_dist>>1);
-				deco->yoff=deco->interl_dist>>1;
+	    			deco->yoff=deco->interl_dist>>1;
 				deco->actual_line=global_cimg
 					->buffer+deco->yoff*3*deco->im_width;
-			}
-		}
-	}
+   			}
+  		}
+ 	}
 }
 
 /* Finds the first char of output string for given codeword. */
@@ -196,7 +162,7 @@ find_first(int c)
 }
 
 /* GIF code
-   Supply a code and it outputs the string for c.
+   Supply a code and it outputs the string for c. 
    if end_callback_hit is set then it should not be called anymore.
 */
 static inline void
@@ -204,13 +170,13 @@ output_string(int c)
 {
 	int pos=0;
 	struct gif_decoder *deco;
-
+ 
 	deco=global_cimg->decoder;
 	while(1){
 		if (pos==4096){
-			/* Cycle in string */
-			end_callback_hit=1;
-			return;
+	  		/* Cycle in string */
+		  	end_callback_hit=1;
+		  	return;
 		}
 		deco->table[pos].garbage=deco->table[c].end_char;
 		pos++;
@@ -226,19 +192,18 @@ output_string(int c)
 
 /* Adds to the code table
    return value: 0 ok
-		 1 fatal error
+                 1 fatal error
 		 2 stop sending data
 */
 
 static inline void
-add_table(struct gif_decoder *deco,int end_char,short pointer)
+add_table(struct gif_decoder *deco,int end_char,int pointer)
 {
 	if (deco->table_pos>=4096){
-		/* ignore overflows because they may happen */
-		/*end_callback_hit=1;*/
-		return; /* Overflow */
-	}
-	deco->table[deco->table_pos].end_char=(unsigned char)end_char;
+	 	end_callback_hit=1;
+	 	return; /* Overflow */
+ 	}
+	deco->table[deco->table_pos].end_char=end_char;
 	deco->table[deco->table_pos].pointer=pointer;
 	deco->table_pos++;
 	if (deco->table_pos==(1<<deco->code_size)&&deco->code_size!=12)
@@ -257,19 +222,19 @@ static inline void
 accept_code(struct gif_decoder *deco,int c)
 {
  int k;
-
- if (c >= 4096 || c < 0) return; /* Erroneous code word will be ignored */
+ 
+ if (c>4096||c<0) return; /* Erroneous code word will be ignored */
  if (c==deco->CC) {
   init_table();
   return;
  }
-
+ 
  if (c==deco->EOI)
  {
   end_callback_hit=1;
   return;
  }
-
+ 
  if (deco->first_code)
  {
   deco->first_code=0;
@@ -280,7 +245,7 @@ accept_code(struct gif_decoder *deco,int c)
   deco->last_code=c;
   return;
  }
-
+ 
  if (c>=deco->table_pos)
  {
   /* The code is not in the table */
@@ -291,10 +256,10 @@ accept_code(struct gif_decoder *deco,int c)
   /* The code is in code table */
   k=find_first(c);
  }
-
+  
  add_table(deco,k,deco->last_code);
  if (end_callback_hit) return;
-
+ 
  /* Output the string for code */
  output_string(c);
  if (end_callback_hit) return;
@@ -311,9 +276,9 @@ accept_byte(unsigned char c)
 {
  int original_code_size;
  struct gif_decoder *deco;
-
+ 
  deco=global_cimg->decoder;
- deco->read_code|=(int)((unsigned long)c<<deco->bits_read);
+ deco->read_code|=(unsigned long)c<<deco->bits_read;  
  deco->bits_read+=8;
  while (deco->bits_read>=deco->code_size)
  {
@@ -331,11 +296,13 @@ accept_byte(unsigned char c)
  * and the buffer must be formatted. */
 static void implant_transparent(struct gif_decoder *deco)
 {
-	if (deco->transparent >= 0 && deco->transparent < (1 << deco->im_bpp)) {
-		if (global_cimg->strip_optimized) {
-			compute_background_8(global_cimg, deco->color_map + 3 * deco->transparent);
-		} else {
-			memcpy(deco->color_map + 3 * deco->transparent, deco->actual_line, 3);
+	if (deco->transparent>=0&&deco->transparent<(1<<deco->im_bpp)){
+		if (global_cimg->strip_optimized){
+			compute_background_8(deco->color_map+3*deco->transparent,
+				global_cimg);	
+		}else{
+			memcpy(deco->color_map+3*deco->transparent
+				,deco->actual_line,3);
 	        }
 	}
 }
@@ -350,19 +317,19 @@ static int gif_dimensions_known(void)
 	global_cimg->width=deco->im_width;
 	global_cimg->height=deco->im_height;
 	global_cimg->red_gamma=global_cimg->green_gamma
-		=global_cimg->blue_gamma=(float)sRGB_gamma;
+		=global_cimg->blue_gamma=sRGB_gamma;
 	global_cimg->strip_optimized=(deco->interl_dist==1
 		&&deco->im_width*deco->im_height>=65536);
 	/* Run strip_optimized only from 65536 pixels up */
 	return header_dimensions_known(global_cimg);
 }
-
+	
 /* Accepts one byte from GIF codestream
    Caller is responsible for destorying the decoder when
    end_callback_hit is 1.
 */
-static inline void
-gif_accept_byte(unsigned char c)
+static inline void 
+gif_accept_byte(int c)
 {
 	struct gif_decoder *deco;
 
@@ -374,10 +341,11 @@ gif_accept_byte(unsigned char c)
 		deco->tbuf[deco->tlen]=c;
 		deco->tlen++;
 		if (deco->tlen>=13){
-			if (strncmp(cast_const_char deco->tbuf,"GIF87a",6)
-				&&strncmp(cast_const_char deco->tbuf,"GIF89a",6)){
-				end_callback_hit=1;
-				return; /* Invalid GIF header */
+			if (strncmp(deco->tbuf,"GIF87a",6)
+				&&strncmp(deco->tbuf,"GIF89a",6)){
+				bad_file:
+	   			end_callback_hit=1;
+	   			return; /* Invalid GIF header */
 			}
 			deco->im_bpp=(deco->tbuf[10]&7)+1;
 			deco->tlen=0;
@@ -389,9 +357,10 @@ gif_accept_byte(unsigned char c)
 				alloc_color_map(1<<deco->im_bpp);
 				deco->state=1;
 			}
-			/*if (deco->tbuf[10] & 8 || deco->tbuf[12]) {
-				goto bad_file;
-			}*/
+			if (deco->tbuf[10] & 8 || deco->tbuf[12]) {
+				/* Test for corrupted file */
+	   			goto bad_file;
+			}
 		}
 		break;
 
@@ -403,7 +372,7 @@ gif_accept_byte(unsigned char c)
 			deco->tlen=0;
 		}
 		break;
-
+  
 		case 2: /* Reading garbage before and one ',' or '!' in GIF */
 		switch (c){
 			case ',':
@@ -415,7 +384,7 @@ gif_accept_byte(unsigned char c)
 			deco->state=3;
 			deco->tlen=0;
 			break;
-
+   
 			case '!':
 			deco->state=7;
 			break;
@@ -434,7 +403,7 @@ gif_accept_byte(unsigned char c)
 			}
 			if (deco->tbuf[8]&64){
 				/* Interlaced order */
-				deco->interl_dist=24; /* Actually 8, the 16 indicates
+				deco->interl_dist=24; /* Actually 8, the 16 indicates 
 						       * it is the first pass. */
 			}else
 				deco->interl_dist=1;
@@ -445,7 +414,7 @@ gif_accept_byte(unsigned char c)
 			if (global_cimg->width && (unsigned)global_cimg->width * (unsigned)global_cimg->buffer_bytes_per_pixel / (unsigned)global_cimg->width != (unsigned)global_cimg->buffer_bytes_per_pixel) overalloc();
 			if ((unsigned)global_cimg->width * (unsigned)global_cimg->buffer_bytes_per_pixel > MAXINT) overalloc();
 			deco->actual_line=global_cimg->strip_optimized
-				?mem_alloc((size_t)global_cimg->width*global_cimg
+				?mem_alloc(global_cimg->width*global_cimg
 				->buffer_bytes_per_pixel)
 				:global_cimg->buffer;
 			if (deco->tbuf[8]&128){
@@ -459,7 +428,7 @@ gif_accept_byte(unsigned char c)
 				deco->xoff=0;
 				deco->yoff=0;
 			}
-		}
+	 	}	 
 		break;
 
 		case 4: /* Reading local colormap in GIF */
@@ -474,15 +443,15 @@ gif_accept_byte(unsigned char c)
 
 		case 5: /* Reading code size block in GIF */
 		deco->initial_code_size=c;
-		if (deco->initial_code_size<=1||deco->initial_code_size>8){
+  		if (deco->initial_code_size<=1||deco->initial_code_size>8){
 			end_callback_hit=1;
 			return; /* Invalid initial code size */
 		}
 		if (!deco->color_map){
-			end_callback_hit=1;
-			return;
-		}
-		deco->bits_read=0;
+	 		end_callback_hit=1;
+	 		return;
+  		}
+  		deco->bits_read=0;
 		deco->read_code=0;  /* Nothing read */
 		init_table(); /* Decoding is about to begin sets up code_size. */
 		deco->state=6;
@@ -492,7 +461,7 @@ gif_accept_byte(unsigned char c)
 		break;
 
 		case 6: /* Reading image data in GIF */
-		if (!deco->remains){
+	  	if (!deco->remains){
 			/* This byte is a count byte. Feed it into remains. */
 			deco->remains=c;
 			if (!c){
@@ -500,7 +469,7 @@ gif_accept_byte(unsigned char c)
     end_callback_hit=1; /* Don't send any following data */
     return;
    }
-  }
+  } 
   else
   {
    accept_byte(c);
@@ -508,7 +477,7 @@ gif_accept_byte(unsigned char c)
    deco->remains--;
   }
   break;
-
+  
   case 7: /* Reading a byte after '!' in GIF */
   if (c==0xf9) deco->state=9;
   else deco->state=8;
